# Copyright (C) 2008 Mark A. Matienzo
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with worldcat.  If not, see <http://www.gnu.org/licenses/>.

""" worldcat/request/xid.py -- Request objects for xID APIs

    xID APIs as of this writing include xISBN, xISSN, and xOCLCNUM.
    
    'Alternate request formats' (such as OpenURL and unAPI) have not been
    implemented.
    
"""

from worldcat.request import WorldCatRequest
from worldcat.response.xid import xIDResponse

class xIDRequest(WorldCatRequest):
    """request.xid.xIDRequest: Base class for requests from xID APIs.
    
    All xIDRequests require a record number ('rec_num') to be passed when
    a class is instantiated. Depending on the request, this will either be
    an ISBN, an ISSN, or an OCLC record number.
    
    xIDRequests by default have their 'method' set as 'getEditions' and their
    response format set as 'python'.
    
    """
    def __init__(self, rec_num=None, **kwargs):
        """Constructor for xIDRequests."""
        if 'method' not in kwargs:
            kwargs['method'] = 'getEditions'
        if 'format' not in kwargs:
            kwargs['format'] = 'python'
        if 'fl' not in kwargs:
            kwargs['fl'] = '*'
        WorldCatRequest.__init__(self, **kwargs)
        self.rec_num = rec_num
        
    def get_response(self):
        self.http_get()
        return xIDResponse(self)
        
    def subclass_validator(self, quiet=False):
        """Validator method for xIDRequests.

        Does not validate ISSN or ISBN values; this should be handled
        by the xID APIs.
        
        """
        if self.rec_num == None:
            if quiet == True:
                return False
            else:
                raise EmptyRecordNumberError
        else:
            return True

class xISSNRequest(xIDRequest):
    """request.xid.xISSNRequest: Class for xISSN requests
    
    For more information on the xISSN API, see
    <http://xissn.worldcat.org/xissnadmin/doc/api.htm>.
    
    Example of an xISSNRequest:
    
        >>> from worldcat.request.xid import xISSNRequest
        >>> x = xISSNRequest(rec_num='1895-104X')
        >>> x.validate()
        >>> r = x.get_response()
    
    """
    def __init__(self, rec_num=None, **kwargs):
        """Constructor method for xISSNRequests."""
        xIDRequest.__init__(self, rec_num, **kwargs)
        self._validators = {
            'method': ('getForms', 'getHistory', 'fixChecksum',
                        'getMetadata', 'getEditions'),
            'format': ('xml', 'html', 'json', 'python',
                        'ruby', 'text', 'csv', 'php')
            }
        
    def api_url(self):
        self.url = 'http://xissn.worldcat.org/webservices/xid/issn/%s' \
            % self.rec_num
            
class xISBNRequest(xIDRequest):
    """request.xisbn.xISBNRequest: Class for xISBN requests

    """
    def __init__(self, rec_num=None, **kwargs):
        """Constructor method for xISBNRequests."""
        xIDRequest.__init__(self, rec_num, **kwargs)
        self._validators = {
            'method': ('to10', 'to13', 'fixChecksum',
                        'getMetadata', 'getEditions'),
            'format': ('xml', 'html', 'json', 'python',
                        'ruby', 'txt', 'csv', 'php')
            }

    def api_url(self):
        self.url = 'http://xisbn.worldcat.org/webservices/xid/isbn/%s' \
            % self.rec_num
            
class xLCCNRequest(xIDRequest):
    """request.xisbn.xLCCNRequest: Class for xID LCCN requests

    """
    def __init__(self, rec_num=None, **kwargs):
        """Constructor method for xISBNRequests."""
        xIDRequest.__init__(self, rec_num, **kwargs)
        self._validators = {
            'method': ('getVariants', 'getMetadata', 'getEditions'),
            'format': ('xml', 'html', 'json', 'python',
                        'ruby', 'txt', 'csv', 'php')
            }

    def api_url(self):
        self.url = 'http://xisbn.worldcat.org/webservices/xid/lccn/%s' \
            % self.rec_num
            
class xOCLCNUMRequest(xIDRequest):
    """request.xisbn.xOCLCNUMRequest: Class for xOCLCNUM requests

    """
    def __init__(self, rec_num=None, **kwargs):
        """Constructor method for xISBNRequests."""
        xIDRequest.__init__(self, rec_num, **kwargs)
        self._validators = {
            'method': ('getVariants', 'getMetadata', 'getEditions'),
            'format': ('xml', 'html', 'json', 'python',
                        'ruby', 'txt', 'csv', 'php')
            }

    def api_url(self):
        self.url = 'http://xisbn.worldcat.org/webservices/xid/oclcnum/%s' \
            % self.rec_num



# Copyright (C) 2008 Mark A. Matienzo
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with worldcat.  If not, see <http://www.gnu.org/licenses/>.

# request/search.py -- Request objects for WorldCat Search API

import urllib2
from exceptions import StopIteration

from worldcat.request import WorldCatRequest
from worldcat.response.search import SearchAPIResponse
from worldcat.util.extract import extract_elements

class SearchAPIRequest(WorldCatRequest):
    """request.search.SearchAPIRequest: base class for all search API requests
    
    SearchAPIRequests require an API key when an instance is created. This is
    done by passing the 'wskey' kwarg. E.g.:
    
        >>> s = SearchAPIRequest(wskey='...insert your api key here')
    
    """
    def __init__(self, **kwargs):
        """Constructor for SearchAPIRequest"""
        if 'wskey' not in kwargs:
            raise APIKeyNotSpecifiedError
        WorldCatRequest.__init__(self, **kwargs)
        self._validators = {
            'servicelevel': ('default', 'full'),
            'cformat': ('apa', 'chicago', 'harvard', 'mla', 'turabian'),
            'recordSchema': ('info:srw/schema/1/marcxml-v1.1',
                                'info:srw/schema/1/dc-v1.1'),
            'format': ('atom','rss')
        }
    
    def get_response(self):
        """Get method for SearchAPIRequests.
        
        Exception handling is specific to SearchAPIRequests.
        """
        try:
            self.http_get()
        except urllib2.HTTPError, e:
            if e.code == 407:
                raise APIKeyError
            elif e.code == 400:
                raise APIKeyNotSpecifiedError
            else:
                raise
        return SearchAPIResponse(self)

class OpenSearchRequest(SearchAPIRequest):
    """request.search.OpenSearchRequest: queries search API using OpenSearch
    
    OpenSearchRequests are always keyword searches."""
    def __init__(self, **kwargs):
        """Constructor for OpenSearch requests"""
        SearchAPIRequest.__init__(self, **kwargs)

    def __iter__(self):
        return self
        
    def api_url(self):
        """API ase URL method for OpenSearchRequests."""
        self.url = 'http://worldcat.org/webservices/catalog/search/opensearch'
    
    def next(self):
        _i = extract_elements(self.response,
                element='{http://a9.com/-/spec/opensearch/1.1/}startIndex')
        _p = extract_elements(self.response,
                element='{http://a9.com/-/spec/opensearch/1.1/}itemsPerPage')
        _t = extract_elements(self.response,
                element='{http://a9.com/-/spec/opensearch/1.1/}totalResults')
        try:
            if int(_t[0].text) > (int(_i[0].text) + int(_p[0].text)):
                self.args['start'] = int(_i[0].text) + int(_p[0].text)
            else:                  
                raise StopIteration
        except ValueError:
            raise StopIteration
        
    def subclass_validator(self, quiet=False):
        """Validator method for OpenSearchRequests."""
        if 'q' not in self.args:
            if quiet == True:
                return False
            else:
                raise EmptyQueryError
        else:
            return True

class SRURequest(SearchAPIRequest):
    """request.search.SRURequest: queries search API using SRU
    
    SRURequests should be used when fielded searching is desired.
    """
    def __init__(self, **kwargs):
        """Constructor method for SRURequests."""
        SearchAPIRequest.__init__(self, **kwargs)

    def __iter__(self):
        return self

    def api_url(self):
        self.url = 'http://worldcat.org/webservices/catalog/search/sru'
    
    def next(self):
        _i = extract_elements(self.response,
                element='{http://www.loc.gov/zing/srw/}nextRecordPosition')
        if len(_i) != 0:
            if _i[0].text is not None:
                self.args['startRecord'] = int(_i[0].text)
            else:
                raise StopIteration
        else:
            raise StopIteration

    def subclass_validator(self, quiet=False):
        """Validator method for SRURequests."""
        if 'query' not in self.args:
            if quiet == True:
                return False
            else:
                raise EmptyQueryError
        else:
            return True
                        
class ContentRequest(SearchAPIRequest):
    """request.search.ContentRequest: search API content request metaclass
    
    ContentRequests are always for an individual record number; they must have
    rec_num as a mandatory parameter when an instance is created. 
    
    """
    def __init__(self, rec_num, **kwargs):
        """Constructor method for ContentRequests."""
        SearchAPIRequest.__init__(self, **kwargs)
        self.rec_num = rec_num
    
    def subclass_validator(self, quiet=False):
        """Validator method for ContentRequests."""
        if self.rec_num is None:
            if quiet == True:
                return False
            else:
                raise EmptyRecordNumberError
        else:
            return True

class BibRequest(ContentRequest):
    """request.search.BibRequest: retrieves single bibliographic records
    
    BibRequests only provide SearchAPIResponses where response_format is
    'xml' and record_format is 'marcxml'.
    
    """
    def __init__(self, rec_num=None, **kwargs):
        """Constructor for BibRequests."""
        ContentRequest.__init__(self, rec_num, **kwargs)

    def api_url(self):
        self.url = 'http://worldcat.org/webservices/catalog/content/%s' \
            % self.rec_num

class CitationRequest(ContentRequest):
    """request.search.CitationRequest: retrieves formatted HTML citations
    
    CitationRequests should always have a SearchAPIResponse where
    response_format is 'html' and record_format is unset.
    
    TODO: Consider handling citation format."""
    def __init__(self, rec_num=None, **kwargs):
        """Constructor for CitationRequests."""
        ContentRequest.__init__(self, rec_num, **kwargs)
    
    def api_url(self):
        """Get method for CitationRequests."""
        self.url = \
            'http://worldcat.org/webservices/catalog/content/citations/%s' \
            % self.rec_num

class HoldingsRequest(ContentRequest):
    """request.search.HoldingsRequest: retrieves holdings for a single record
    
    HoldingsRequests universally going to have a SearchAPIResponse where
    response_format is 'xml' and record_format is 'iso20775'.
    """
    def __init__(self, rec_num=None, num_type='oclc', **kwargs):
        """Constructor for HoldingsRequests."""
        self._nt_validator = { 'oclc': '', 'isbn': 'isbn/'}
        self.num_type = num_type
        ContentRequest.__init__(self, rec_num, **kwargs)
        
    def api_url(self):
        """Get method for HoldingsRequests."""
        self.url = \
            'http://worldcat.org/webservices/catalog/content/libraries/%s%s' \
            % (NUM_TYPES.get(self._nt_validator), self.rec_num)

    def subclass_validator(self, quiet=False):
        """Validator method for HoldingsRequests.
        
        Despite HoldingsRequests being able to handle ISBNs,
        HoldingsRequest.validate() does not validate ISBNS."""
        if self.num_type not in self._nt_validator:
            if quiet == True:
                return False
            else:
                raise InvalidNumberTypeError
        else:
            return True


# Copyright (C) 2004 Sami Hangaslammi
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with worldcat.  If not, see <http://www.gnu.org/licenses/>.

# util/safeeval.py - Safe evaluation of python constants and expressions

# taken from http://code.activestate.com/recipes/286134/

import dis

_const_codes = map(dis.opmap.__getitem__, [
    'POP_TOP','ROT_TWO','ROT_THREE','ROT_FOUR','DUP_TOP',
    'BUILD_LIST','BUILD_MAP','BUILD_TUPLE',
    'LOAD_CONST','RETURN_VALUE','STORE_SUBSCR'
    ])

_expr_codes = _const_codes + map(dis.opmap.__getitem__, [
    'UNARY_POSITIVE','UNARY_NEGATIVE','UNARY_NOT',
    'UNARY_INVERT','BINARY_POWER','BINARY_MULTIPLY',
    'BINARY_DIVIDE','BINARY_FLOOR_DIVIDE','BINARY_TRUE_DIVIDE',
    'BINARY_MODULO','BINARY_ADD','BINARY_SUBTRACT',
    'BINARY_LSHIFT','BINARY_RSHIFT','BINARY_AND','BINARY_XOR',
    'BINARY_OR',
    ])

def _get_opcodes(codeobj):
    """_get_opcodes(codeobj) -> [opcodes]

    Extract the actual opcodes as a list from a code object

    >>> c = compile("[1 + 2, (1,2)]", "", "eval")
    >>> _get_opcodes(c)
    [100, 100, 23, 100, 100, 102, 103, 83]
    """
    i = 0
    opcodes = []
    s = codeobj.co_code
    while i < len(s):
        code = ord(s[i])
        opcodes.append(code)
        if code >= dis.HAVE_ARGUMENT:
            i += 3
        else:
            i += 1
    return opcodes        

def test_expr(expr, allowed_codes):
    """test_expr(expr) -> codeobj

    Test that the expression contains only the listed opcodes.
    If the expression is valid and contains only allowed codes,
    return the compiled code object. Otherwise raise a ValueError
    """
    try:
        c = compile(expr, "", "eval")
    except:
        raise ValueError, "%s is not a valid expression" % expr
    codes = _get_opcodes(c)
    for code in codes:
        if code not in allowed_codes:
            raise ValueError, "opcode %s not allowed" % dis.opname[code]
    return c


def const_eval(expr):
    """const_eval(expression) -> value

    Safe Python constant evaluation

    Evaluates a string that contains an expression describing
    a Python constant. Strings that are not valid Python expressions
    or that contain other code besides the constant raise ValueError.

    >>> const_eval("10")
    10
    >>> const_eval("[1,2, (3,4), {'foo':'bar'}]")
    [1, 2, (3, 4), {'foo': 'bar'}]
    >>> const_eval("1+2")
    Traceback (most recent call last):
    ...
    ValueError: opcode BINARY_ADD not allowed
    """
    c = test_expr(expr, _const_codes)
    return eval(c)

def expr_eval(expr):
    """expr_eval(expression) -> value

    Safe Python expression evaluation

    Evaluates a string that contains an expression that only
    uses Python constants. This can be used to e.g. evaluate 
    a numerical expression from an untrusted source.

    >>> expr_eval("1+2")
    3
    >>> expr_eval("[1,2]*2")
    [1, 2, 1, 2]
    >>> expr_eval("__import__('sys').modules")
    Traceback (most recent call last):
    ...
    ValueError: opcode LOAD_NAME not allowed
    """
    c = test_expr(expr, _expr_codes)
    return eval(c)


# Copyright (C) 2008 Mark A. Matienzo
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with wo.  If not, see <http://www.gnu.org/licenses/>.

# request/__init__.py - Base request classes for WorldCat APIs

import urllib
import urllib2
from worldcat.exceptions import APIKeyError, APIKeyNotSpecifiedError, \
                         EmptyQueryError, EmptyRecordNumberError, \
                         InvalidArgumentError, InvalidNumberTypeError
from worldcat.response import WorldCatResponse
                         
class WorldCatRequest(object):
    """request.WorldCatRequest: base class for all requests to WorldCat APIs
    
    """
    def __init__(self, **kwargs):
        """Constructor method for WorldCatRequests.
        
        Subclasses should pass kwargs and set their subclass-specific
        _validators from their __init__ methods, e.g.:
        
            class WCSubClass(WorldCatRequest):
                def __init__(self, **kwargs):
                    WorldCatRequest.__init__(self, **kwargs)
                    self._validators = {'breakfast': ('spam', 'ham', 'eggs')}
                    
        """
        self.args = kwargs
        self._validators = {}
        
    def api_url(self):
        """Dummy method to set API URL; should be overridden by subclasses."""
        pass
        
    def get_response(self):
        """Dummy method to get response; should be overrriden by subclasses.
        
        Subclass get_response methods should override this method in part as
        they should return the relevant subclass of WorldCatResponse.
        """
        self.http_get()
        return WorldCatResponse(self)
        
    def http_get(self):
        """HTTP Get method for all WorldCatRequests."""
        self.api_url()
        _query_url = '%s?%s' % (self.url, urllib.urlencode(self.args))
        self.response = urllib2.urlopen(_query_url).read()
        
    def subclass_validator(self, quiet):
        """Dummy validator method; should be overriden by subclasses.
        
        Subclasses should use the subclass_validator method to perform
        any validation that doesn't rely on the _validators dict object, e.g.:
        
        class WCSubClass(WorldCatRequest):
            def __init__ ...
            def validate(self):
                if 'argument' not in self.args:
                    raise MissingArgException
                elif self.args['argument'] == 'quux':
                    raise QuuxException
        """
        pass
        
    def validate(self, quiet=False):
        """Validate arguments using a dict of valid values for each argument.

           Validators are specified as dicts with arguments as keys,
           associated with tuples of valid values.
            
           The validate method should be called before the get_response
           method.
           
        """
        subvalid = self.subclass_validator(quiet)
        for key in self._validators:
            if key in self.args:
                if (self.args[key] in self._validators[key]):
                    pass
                elif quiet == True:
                    return False
                else:
                    raise InvalidArgumentError(key, self.args[key])
            else:
                pass
        
        if quiet == True:
            if subvalid == True:
                return True
            else:
                return False
        else:
            pass


# Copyright (C) 2008 Mark A. Matienzo
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with worldcat.  If not, see <http://www.gnu.org/licenses/>.

# response/xid.py - Response objects for xID API requests

from worldcat.response import WorldCatResponse

class xIDResponse(WorldCatResponse):
    """response.xid.xIDResponse: Response class for xID APIs.
    
    xIDRequests can specify Python objects as a response format. The 
    xIDRequest constructor accordingly uses worldcat.util.safeeval to
    determine if the API's response is a valid Python constant object so
    arbitrary code is not run.
    
    xID APIs as of this writing include xISBN, xISSN, and xOCLCNUM.
    
    """
    def __init__(self, _r=None):
        """Constructor for xIDResponses"""
        WorldCatResponse.__init__(self, _r)
        self.response_format = _r.args['format']
        self.method = _r.args['method']
        if (self.response_format == 'python'):
            self.safe_eval()


# Copyright (C) 2008 Mark A. Matienzo
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with worldcat.  If not, see <http://www.gnu.org/licenses/>.

# response/__init__.py - Contains response objects for WorldCat API requests

import worldcat.util.safeeval

class WorldCatResponse(object):
    """response.WorldCatResponse: Base class for responses from WorldCat APIs
    
    """
    def __init__(self, _r):
        """Constructor for WorldCatResponses"""
        self.data = _r.response
        self.eval = False
        self.response_type = _r.__class__.__name__
        
    def safe_eval(self):
        """Only eval a response if self.data is an instance of _obj"""
        try:
            self.data = worldcat.util.safeeval.const_eval(self.data)
            self.eval = True
        except:
            warnings.warn("Response does not eval safely", RuntimeWarning)


# Copyright (C) 2008 Mark A. Matienzo
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with worldcat.  If not, see <http://www.gnu.org/licenses/>.

# response/search.py - Response objects for WorldCat Search API requests

from worldcat.response import WorldCatResponse
from worldcat.util.extract import extract_elements

class SearchAPIResponse(WorldCatResponse):
    """response.search.SearchAPIResponse: WorldCat Search API response class
    
    """
    def __init__(self, _r=None):
        """Constructor for SearchAPIResponses"""
        WorldCatResponse.__init__(self, _r)
        self.response_format = 'xml'
        if self.response_type == 'HoldingsRequest':
            self.record_format = 'iso20775'
        elif self.response_type in ('OpenSearchRequest', 'CitationRequest'):
            if self.response_type == 'CitationRequest':
                self.response_format = 'html'
            elif 'format' in _r.args:
                self.record_format = _r.args['format']
            else:
                self.record_format = 'atom'
        elif self.response_type in ('BibRequest', 'SRURequest'):
            self.record_format = 'marcxml'
            if 'recordSchema' in _r.args:
                if _r.args['recordSchema'] == 'info:srw/schema/1/dc':
                    self.record_format = 'dc'


# Copyright (C) 2008 Mark A. Matienzo
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with worldcat.  If not, see <http://www.gnu.org/licenses/>.

# util/extract.py - Extract elements based upon namespaces

from cStringIO import StringIO

try:
    import xml.etree.ElementTree as ET  # builtin in Python 2.5
except ImportError:
    import elementtree.ElementTree as ET

import pymarc

from worldcat.exceptions import ExtractError

def extract_elements(xml, element='{http://www.loc.gov/MARC21/slim}record'):
    """worldcat.util.extract_elements: extract elements based on namespace
    
    This function will probably prove useful to anyone using SRURequests.
    """
    tree = ET.fromstring(xml)
    return tree.getiterator(element)

def pymarc_extract(xml):
    """worldcat.util.pymarc_extract: extract records to pymarc Record objects
    
    Requires pymarc >= 1.2. StringIO is used since xml.sax.XMLReader's
    parse objects (which pymarc.marcxml.parse_xml uses) expect a filename, a
    file-like object, or an InputSource object. 
    """
    pymarc_records = []
    records = extract_elements(xml)
    for record in records:
        handler = pymarc.XmlHandler()
        pymarc.parse_xml(StringIO(ET.tostring(record)), handler)
        pymarc_records.extend(handler.records)
    return pymarc_records


from setuptools import setup, find_packages

install_requires = []

classifiers = """
Intended Audience :: Education
Intended Audience :: Developers
Intended Audience :: Information Technology
License :: OSI Approved :: BSD License
License :: OSI Approved :: GNU General Public License (GPL)
Programming Language :: Python
Development Status :: 3 - Alpha 
"""

setup( 
    name = 'worldcat',
    version = '0.1.3',  # remember to update worldcat/__init__.py on release!
    url = 'http://matienzo.org/project/worldcat',
    author = 'Mark A. Matienzo',
    author_email = 'mark@matienzo.org',
    license = 'GPL/BSD',
    packages = find_packages(),
    install_requires = install_requires,
    description = 'Interact with OCLC\'s WorldCat Search and xID APIs',
    classifiers = filter(None, classifiers.split('\n')),
)



# Copyright (C) 2008 Mark A. Matienzo
# 
# This file is part of worldcat, the Python WorldCat API module.
# 
# worldcat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# worldcat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with worldcat.  If not, see <http://www.gnu.org/licenses/>.

# exceptions.py - Errors for WorldCat API module

class ExtractError(Exception):
    """exceptions.ExtractError: Exceptions for worldcat.util.extract
    """
    def __init__(self, msg):
        self.msg = msg
    
    def __str__(self):
        return self.msg

class WorldCatAPIError(Exception):
    """exceptions.WorldCatAPIError: Base class for all WorldCat API exceptions
    
    """
    pass
    
class APIKeyError(WorldCatAPIError):
    """exceptions.APIKeyError: General API key exception"""
    def __str__(self):
        return "Invalid API key"

class APIKeyNotSpecifiedError(APIKeyError):
    """exceptions.APIKeyNotSpecifiedError: For empty API Key values"""
    def __str__(self):
        return "API key not specified"

class ValidationError(WorldCatAPIError):
    """exceptions.ValidationError: Base class for validation errors"""
    pass
    
class EmptyQueryError(ValidationError):
    """exceptions.EmptyQueryError: Query exception for null queries

    for OpenSearchRequests and SRURequests."""
    def __str__(self):
        return "Query terms missing"

class EmptyRecordNumberError(ValidationError):
    """exceptions.EmptyRecordNumberError: For unspecified rec_num values
    
    For BibRequests, CitationRequests, HoldingsRequests, and xIDRequests.
    
    """
    def __str__(self):
        return "Record number for content request missing"

class InvalidNumberTypeError(ValidationError):
    """exceptions.InvalidNumberTypeError: For invalid number type selections
    
    For HoldingsRequsts.
    """
    def __str__(self):
        return "Invalid record number type"

class InvalidArgumentError(ValidationError):
    """exceptions.InvalidArgumentError: For invalid arguments
    
    This relies on the WorldCatRequest._validators dict; see docstring for
    request.WorldCatRequest.__init__ for more information.
    """
    def __init__(self, arg, value):
        self.arg = arg
        self.value = value

    def __str__(self):
        return "API argument '%s' has invalid value '%s'" % (self.arg, self.value)


"""find related things in worldcat by oclcnumber"""
from worldcat.request.search import BibRequest, CitationRequest, SRURequest
from worldcat.util.extract import extract_elements, pymarc_extract

import web

import os
import sys

urls = ('/', 'index', '/related/(.*)', 'related', '/related', 'related')
render = web.template.render('templates/')
WSKEY = sys.argv[1]

def get_eds(oclcnum):
    bib = BibRequest(wskey=WSKEY, rec_num=oclcnum).get_response()
    record = pymarc_extract(bib.data)
    subjects = []
    if len(record) == 0:
        return 'record %s not found' % oclcnum
    else:
        # if record[0].author() is not None:
        #         subjects.append(record[0].author())
        for s in record[0].subjects():
            if s.indicator2 == '0':
                subjects.append('"%s"' % s.format_field())
            elif s.indicator2 == '7' and s.tag == '655':
                subjects.append('"%s"' % s['a'])
        # if len(subjects) > 5:
        #     subjects = subjects[0:5]
        #     print 'more than 5 subjects - truncating'
        #print subjects
        # 'not srw.no = oclcnum'
        if len(subjects) > 0:
            sru = SRURequest(wskey=WSKEY)
            sru.args['query'] = '(srw.kw = %s) not srw.no = "%s"' % \
                                    (" or srw.kw = ".join(subjects), oclcnum)
            # sru.args['servicelevel'] = 'full'
            sru.args['sortKeys'] = 'LibraryCount,,0 relevance'
            #print sru.args['query']
            results = pymarc_extract(sru.get_response().data)
            results = [r['001'].format_field() for r in results]
            out = []
            for r in results:
                c = CitationRequest(wskey=WSKEY, rec_num=r)
                out.append(c.get_response().data)
            return "".join(out)
        else:
            return None

class index:
    def GET(self):
        print render.index()

class related:
    def GET(self, oclcnum):
        numcite = CitationRequest(wskey=WSKEY, rec_num=oclcnum).get_response().data
        print render.related(oclcnum=oclcnum, numcite=numcite,
                                response=get_eds(oclcnum))
    
    def POST(self):
        oclcnum = web.input().oclcnum
        numcite = CitationRequest(wskey=WSKEY, rec_num=oclcnum).get_response().data
        print render.related(oclcnum=oclcnum, numcite=numcite,
                                response=get_eds(oclcnum))

web.webapi.internalerror = web.debugerror
if __name__ == '__main__':
    web.run(urls, globals())